#!/usr/bin/env node

const debug = require("debug")("ldi");

const os = require("os");
const program = require("commander");
const fs = require("fs");
const path = require("path");
const spawn = require("child_process").spawn;
const spawnSync = require("child_process").spawnSync;
const git = require("git-state");
const gitu = require("git-utils");

const ldiModules = [
  "util",
  "param",
  "account",
  "common",
  "wallet",
  "payment-bni",
  "loan",
  "investment",
  "report",
  "jsreport"
];

let base;
if (git.isGitSync(".")) {
  base = path.resolve("..");
} else {
  base = path.resolve(".");
}
for (const mod of ldiModules) {
  const fn = path.join(base, mod);
  if (!fs.existsSync(fn)) {
    console.log(`Module ${mod} not found in ${fn}`);
    process.exit(1);
  }
}

function gitStatus() {
  for (const mod of ldiModules) {
    const fn = path.join(base, mod);
    const gst = git.checkSync(fn);
    const repo = gitu.open(fn);
    console.log(`${mod}: ${gst.branch}`);
    let clean = true;
    const abc = repo.getAheadBehindCount();
    if (abc.ahead !== 0) {
      clean = false;
      console.log(`   Ahead:     ${abc.ahead}`);
    }
    if (abc.behind !== 0) {
      clean = false;
      console.log(`   Behind:    ${abc.behind}`);
    }
    if (gst.dirty !== 0) {
      clean = false;
      console.log(`   Dirty:     ${gst.dirty}`);
    }
    if (gst.untracked !== 0) {
      clean = false;
      console.log(`   Untracked: ${gst.untracked}`);
    }
    if (clean) {
      console.log(`   CLEAN`);
    }
  }
}

program
  .command("status")
  .description("git status all modules")
  .action(() => {
    gitStatus();
  });

program
  .command("fetch")
  .description("git fetch all modules")
  .action(() => {
    for (const mod of ldiModules) {
      const fn = path.join(base, mod);
      console.log(`${mod}:`);
      spawnSync("git", ["fetch", "--all"], {
        cwd: fn,
        stdio: "inherit"
      });
    }
    gitStatus();
  });

program.on("command:*", () => {
  program.help();
  process.exit(1);
});

if (process.argv.length <= 2) {
  program.help();
  process.exit(1);
}

program.parse(process.argv);
